@@ -254,6 +254,7 @@ pango_ft2_pattern_equal (MiniXftPattern *pattern1,
 static void 
 pango_ft2_font_map_init (PangoFT2FontMap *ft2fontmap)
 {
+  ft2fontmap->n_families = -1;
 
   ft2fontmap->fonts =
     g_hash_table_new ((GHashFunc)pango_ft2_pattern_hash,
@@ -402,6 +403,7 @@ pango_ft2_font_map_list_families (PangoFontMap           *fontmap,
 	  ft2fontmap->families[i] = g_object_new (PANGO_FT2_TYPE_FAMILY, NULL);
 	  ft2fontmap->families[i]->family_name = g_strdup (s);
 	  ft2fontmap->families[i]->fontmap = ft2fontmap;
+          ft2fontmap->families[i]->n_faces = -1;
 	}
 
       MiniXftFontSetDestroy (fontset);
@@ -543,6 +545,9 @@ pango_ft2_font_map_load_fontset (PangoFontMap                 *fontmap,
     {
       pattern = pango_ft2_make_pattern (desc);
 
+      MiniXftInit (0);
+      MiniXftInitFtLibrary ();
+	  
       MiniXftConfigSubstitute (pattern);
       MiniXftDefaultSubstitute ((Display *)1, 0, pattern);
 
@@ -551,9 +556,6 @@ pango_ft2_font_map_load_fontset (PangoFontMap                 *fontmap,
       array = g_ptr_array_new ();
       patterns = g_new (PangoFT2PatternSet, 1);
 
-      MiniXftInit (0);
-      MiniXftInitFtLibrary ();
-	  
       match = NULL;
       id = 0;
       while (MiniXftPatternGetString (pattern, XFT_FAMILY, id++, &family) == MiniXftResultMatch)
@@ -653,11 +655,13 @@ pango_ft2_font_map_cache_clear (PangoFT2FontMap *ft2fontmap)
  */
 
 PangoFontDescription *
-_pango_ft2_font_desc_from_pattern (MiniXftPattern *pattern)
+_pango_ft2_font_desc_from_pattern (MiniXftPattern *pattern,
+				   gboolean        include_size)
 {
   PangoFontDescription *desc;
   PangoStyle style;
   PangoWeight weight;
+  double size;
   
   char *s;
   int i;
@@ -700,6 +704,9 @@ _pango_ft2_font_desc_from_pattern (MiniXftPattern *pattern)
   else
     weight = PANGO_WEIGHT_NORMAL;
 
+  if (include_size && MiniXftPatternGetDouble (pattern, XFT_SIZE, 0, &size) == MiniXftResultMatch)
+      pango_font_description_set_size (desc, size * PANGO_SCALE);
+  
   pango_font_description_set_weight (desc, weight);
   
   pango_font_description_set_variant (desc, PANGO_VARIANT_NORMAL);
@@ -730,7 +737,7 @@ pango_ft2_face_describe (PangoFontFace *face)
   result_pattern = MiniXftFontMatch ((Display *)1, 0, match_pattern, &res);
   if (result_pattern)
     {
-      desc = _pango_ft2_font_desc_from_pattern (result_pattern);
+      desc = _pango_ft2_font_desc_from_pattern (result_pattern, FALSE);
       MiniXftPatternDestroy (result_pattern);
     }
 
