@@ -1,5 +1,5 @@
 /* Pango
- * pangoxft-fontmap.h: Xft font handling
+ * pangoxft-fontmap.c: Xft font handling
  *
  * Copyright (C) 2000 Red Hat Software
  *
@@ -516,16 +516,21 @@ pango_xft_font_map_load_font (PangoFontMap               *fontmap,
   PangoXftFontMap *xfontmap = (PangoXftFontMap *)fontmap;
   XftPattern *pattern, *match;
   XftResult res;
+  PangoFont *font = NULL;
   
   pattern = pango_xft_make_pattern (description);
       
   match = XftFontMatch (xfontmap->display, xfontmap->screen, pattern, &res);
+  
   XftPatternDestroy (pattern);
   
   if (match)
-    return  pango_xft_font_map_new_font (fontmap, match);
-
-  return NULL;
+    {
+      font = pango_xft_font_map_new_font (fontmap, match);
+      XftPatternDestroy (match);
+    }
+  
+  return font;
 }
 
 
@@ -720,11 +725,12 @@ _pango_xft_font_map_get_info (PangoFontMap *fontmap,
  */
 
 PangoFontDescription *
-_pango_xft_font_desc_from_pattern (XftPattern *pattern)
+_pango_xft_font_desc_from_pattern (XftPattern *pattern, gboolean include_size)
 {
   PangoFontDescription *desc;
   PangoStyle style;
   PangoWeight weight;
+  double size;
   
   char *s;
   int i;
@@ -767,6 +773,9 @@ _pango_xft_font_desc_from_pattern (XftPattern *pattern)
   else
     weight = PANGO_WEIGHT_NORMAL;
 
+  if (include_size && XftPatternGetDouble (pattern, XFT_SIZE, 0, &size) == XftResultMatch)
+      pango_font_description_set_size (desc, size * PANGO_SCALE);
+  
   pango_font_description_set_weight (desc, weight);
   
   pango_font_description_set_variant (desc, PANGO_VARIANT_NORMAL);
@@ -797,7 +806,7 @@ pango_xft_face_describe (PangoFontFace *face)
   result_pattern = XftFontMatch (xfontmap->display, xfontmap->screen, match_pattern, &res);
   if (result_pattern)
     {
-      desc = _pango_xft_font_desc_from_pattern (result_pattern);
+      desc = _pango_xft_font_desc_from_pattern (result_pattern, FALSE);
       XftPatternDestroy (result_pattern);
     }
 
